#include "stdafx.h"
#include "clf_math.h"

///////////////////////////////////////////////////////////////////////////////
// custom trigonometry
#define DEG_DIVS   10.f
#define CIRCLE_DIV 3600
static float cosines[CIRCLE_DIV];
static float sines[CIRCLE_DIV];
static float tangents[CIRCLE_DIV];

static void init_trig()
{
	int i;
	for (i=0; i<CIRCLE_DIV; i++)
	{
		float a = ((float)i*pi/(float(CIRCLE_DIV)/2.f));
		cosines[i]  = cos(a);
		sines[i]    = sin(a);
		tangents[i] = tan(a);
	}
}

float myCos(float a)
{
	a *= DEG_DIVS;
	while (a<0)
		a += (float)CIRCLE_DIV;
	return cosines[(int)a];
}

float mySin(float a)
{
	a *= DEG_DIVS;
	while (a<0)
		a += (float)CIRCLE_DIV;
	while (a>CIRCLE_DIV)
		a -= (float)CIRCLE_DIV;
	return sines[(int)a];
}

float myTan(float a)
{
	a *= DEG_DIVS;
	while (a<0)
		a += (float)CIRCLE_DIV;
	while (a>CIRCLE_DIV)
		a -= (float)CIRCLE_DIV;
	return tangents[(int)a];
}

///////////////////////////////////////////////////////////////////////////////
// custom math

// check if the two line segments (p1, p2) and (p3,p4) intersect
bool intersect(const clf_point2D& p1, const clf_point2D& p2, const clf_point2D& p3, const clf_point2D& p4, clf_point2D* p0)
{
	// first check horizontal and vertical gap
	if (min(p1.x, p2.x) > max(p3.x,p4.x))
		return false;
	if (max(p1.x, p2.x) < min(p3.x,p4.x))
		return false;
	if (min(p1.y, p2.y) > max(p3.y,p4.y))
		return false;
	if (max(p1.y, p2.y) < min(p3.y,p4.y))
		return false;

	// firsty calculate the common denominator
	float d = (p4.y-p3.y)*(p2.x-p1.x)-(p4.x-p3.x)*(p2.y-p1.y);
	if (fabs(d) < 0.0000001) // almost 0 (compensate for computer accuracy)
		return false;

	// calculate fractional part for first segment
	float na = ((p4.x-p3.x)*(p1.y-p3.y)-(p4.y-p3.y)*(p1.x-p3.x))/d;
	if (na < 0.f || na > 1.f) // beyond limits of first segment, search no more
		return false;

	// calculate fractional part for second segment
	float nb = ((p2.x-p1.x)*(p1.y-p3.y)-(p2.y-p1.y)*(p1.x-p3.x))/d;
	if (nb < 0.f || nb > 1.f) // beyond limits of second segment, search no more
		return false;

	// we have intersection, calculate actual coordinates (if required)
	if (p0)
	{
		p0->x = p1.x+na*(p2.x-p1.x);
		p0->y = p1.y+na*(p2.y-p1.y);
		// or using the second segment endpoints
		// p0->x = p3.x+nb*(p4.x-p3.x);
		// p0->y = p3.y+nb*(p4.y-p3.y);

	}

	// return true
	return true;
}

// check if the point (pt) is internal to the polygon (poly)
bool point_in_polygon(clf_polygon2D& poly, clf_point2D pt)
{
	// first check if the point is out of the bounds of the polygon
	if (pt.x < poly.bounds.left || pt.x > poly.bounds.right || pt.y < poly.bounds.bottom || pt.y > poly.bounds.top)
		return false;

	// generate a point out of the polygon with the same y coordinate as the check point
	clf_point2D right(poly.bounds.right+1.f, pt.y);
	clf_point2D isect;

	// now iterate through the points of the polygon
	clf_polygon2D::iterator next = poly.begin();
	clf_polygon2D::iterator it;
	int count = 0;
	for (it=poly.begin() ; it!=poly.end(); ++it)
	{
		// -next- points to the 'next' point while -it- is the current
		++next;
		if (next == poly.end())
			next = poly.begin();

		if (intersect(*it, *next, pt, right, &isect))
		{
			if (fabs(it->y-isect.y) < 0.001) // the benefit of horizontal lines is that we only need to check the y coordinate
			{
				// ignore it, we will catch it as ending point later
			}
			// if the ending point of the segment is the intersection
			else if (fabs(next->y-isect.y) < 0.001) // the benefit of horizontal lines is that we only need to check the y coordinate
			{
				clf_polygon2D::iterator next_next = next;
				++next_next;
				if (next_next == poly.end())
					next_next = poly.begin();
				// check if the two segments are on the same side of the horizontal line
				if ((next_next->y-next->y)*(next->y-it->y) > 0) // dy1*dy2>0 ==> the polygon intersects
				{
					count++; // boundary crossing, count the intersection
				}
				else
				{
					// not an actual intersection, do not count it
				}
			}
			else
			{ // intersection point, count it
				count ++;
			}
		}
	}

	if (count%2) // odd number of intersections
		return true;

	return false;
}

// check if the two polygons collide
bool polygons_collide(clf_polygon2D& poly1, clf_polygon2D& poly2)
{
	// first check if they are valid
	if (poly1.size() < 2 || poly2.size() < 2)
		return false;
	// then if they are near enough
	if (poly2.bounds.right < poly1.bounds.left || poly2.bounds.left > poly1.bounds.right || 
			poly2.bounds.top < poly1.bounds.bottom || poly2.bounds.bottom > poly1.bounds.top)
		return false;

	// now if there is an intersection between their edges
	clf_polygon2D::iterator it1;
	clf_polygon2D::iterator it2;
	clf_polygon2D::iterator next1=poly1.begin();
	++next1;
	for (it1=poly1.begin(); it1!=poly1.end(); ++it1)
	{
		if (next1 == poly1.end()) next1 = poly1.begin();
		clf_polygon2D::iterator next2=poly2.begin();
		++next2;
		for (it2=poly2.begin(); it2!=poly2.end(); ++it2)
		{
			if (next2 == poly2.end()) next2 = poly2.begin();
			if (intersect(*it1, *next1, *it2, *next2, NULL))
				return true;
			++next2;
		}
		++next1;
	}

	// final check, if we want to cover all bases check weather one polygon is inside the other
	it1=poly1.begin();
	if (point_in_polygon(poly2, *it1)) // one point is enough
		return true;
	it2=poly2.begin();
	if (point_in_polygon(poly1, *it2))
		return true;

	return false;
}

// rotate point (x,y) around center (cx,cy) by (angle)
void rotate2D(float cx, float cy, float *x, float *y, float angle)
{
	float dx = *x - cx;
	float dy = *y - cy;

	*x = cx + dx*cos(angle) - dy*sin(angle);
	*y = cy + dx*sin(angle) + dy*cos(angle);
}

// check for intersection, but exclude the vertices
bool intersect_no_vertex(const clf_point2D& p1, const clf_point2D& p2, const clf_point2D& p3, const clf_point2D& p4, clf_point2D* p0)
{
	// first check horizontal and vertical gap
	if (min(p1.x, p2.x)>max(p3.x,p4.x))
		return false;
	if (max(p1.x, p2.x)<min(p3.x,p4.x))
		return false;
	if (min(p1.y, p2.y)>max(p3.y,p4.y))
		return false;
	if (max(p1.y, p2.y)<min(p3.y,p4.y))
		return false;

	// firsty calculate the common denominator
	float d = (p4.y-p3.y)*(p2.x-p1.x)-(p4.x-p3.x)*(p2.y-p1.y);
	if (fabs(d) < 0.0000001) // almost 0 (compensate for computer accuracy)
		return false;

	// calculate fractional part for first segment
	float na = ((p4.x-p3.x)*(p1.y-p3.y)-(p4.y-p3.y)*(p1.x-p3.x))/d;
	if (na < 0.0000001f || na > 0.9999999f) // beyond limits of first segment, search no more
		return false;

	// calculate fractional part for second segment
	float nb = ((p2.x-p1.x)*(p1.y-p3.y)-(p2.y-p1.y)*(p1.x-p3.x))/d;
	if (nb < 0.0000001f || nb > 0.9999999f) // beyond limits of second segment, search no more
		return false;

	// we have intersection, calculate actual coordinates (if required)
	if (p0)
	{
		p0->x = p1.x+na*(p2.x-p1.x);
		p0->y = p1.y+na*(p2.y-p1.y);
		// or using the second segment endpoints
		// p0->x = p3.x+nb*(p4.x-p3.x);
		// p0->y = p3.y+nb*(p4.y-p3.y);

	}

	return true;
}

// check if the line segment (p1,p2) is inside a polygon
bool line_in_polygon(clf_polygon2D& poly, clf_point2D& p1, clf_point2D& p2)
{
	// the line that connects the first and the third point is entirely inside thepolygon
	clf_point2D pt((p1.x+p2.x)/2, (p1.y+p2.y)/2); // center point
	if (!point_in_polygon(poly, pt)) // if it is outside the polygon
		return false;    // triangle is invalid
	// iterate through the edges of the polygon
	clf_polygon2D::iterator next = poly.begin();
	clf_polygon2D::iterator it;
	int count = 0;
	for (it=poly.begin() ; it!=poly.end(); ++it)
	{
		// -next- points to the 'next' point while -it- is the current
		++next;
		if (next == poly.end())
			next = poly.begin();
		// we know that the vertices are on the polygon!!!
		if (intersect_no_vertex(*it, *next, p1, p2, NULL))
		{
			return false;
		}
	}

	return true;
}

bool triangulate(clf_polygon2D& poly, std::vector<clf_triangle2D>& result)
{
	// clear the output buffer
	result.erase(result.begin(), result.end());
	// generate index buffer, instead of the points we keep their respective indices
	std::list<int> tp; // valid indices
	std::vector<clf_point2D>::iterator it;
	int i=0;
	for (it=poly.begin(); it!=poly.end(); ++it)
	{
		tp.push_back(i);
		i++;
	}

	// and here we go
	clf_triangle2D t;
	std::list<int>::iterator ti;
	while (tp.size() > 3) // while we have a polygon
	{
		ti = tp.begin();    // go to its start
		// and get the first three points
		int i1 = *ti; ++ti;
		int i2 = *ti; ++ti;
		int i3 = *ti; ++ti;
		// check if they form a valid triangle
		// the line that connects the first and the third point is entirely inside the polygon
		bool valid = false; // assume invalid triangle
		// if so, does the line intersect with the polygon?
		if (line_in_polygon(poly, poly[i1], poly[i3]))
		{
			// if it is inside we have a valid triangle
			t.verts[0] = poly[i1];
			t.verts[1] = poly[i2];
			t.verts[2] = poly[i3];
			result.push_back(t); // store it
			valid = true;   // and prepare for the next
		}
		ti = tp.begin(); // back to the start of the index buffer
		if (valid)        // if the triangle was valid move to the second point
			++ti;
		int tpi = *ti;   // get the index of the current point (first or second)
		tp.erase(ti);    // remove the point from the buffer
		if (!valid)       // if the triangle was invalid
			tp.push_back(tpi);  // store the first point at the end of the point buffer
	}
	// now store the remaining triangle
	{
	ti = tp.begin();
	int i1 = *ti; ++ti;
	int i2 = *ti; ++ti;
	int i3 = *ti; ++ti;
	t.verts[0] = poly[i1];
	t.verts[1] = poly[i2];
	t.verts[2] = poly[i3];
	result.push_back(t);
	}

	return true;
}


///////////////////////////////////////////////////////////////////////////////
//

clf_vector3D& clf_vector3D::rotate(clf_matrix4x4& mMatrix)
{
	clf_vector3D vOut;
	vOut[0] = mMatrix[0] * x + mMatrix[4] * y + mMatrix[8] *  z;
	vOut[1] = mMatrix[1] * x + mMatrix[5] * y + mMatrix[9] *  z;
	vOut[2] = mMatrix[2] * x + mMatrix[6] * y + mMatrix[10] * z;
	memcpy(&x, vOut, 3*sizeof(float));
	return *this;
}

///////////////////////////////////////////////////////////////////////////////

void clf_matrix4x4::loadIdentity()
{
	static float identity[] = { 1.0f, 0.0f, 0.0f, 0.0f,
								0.0f, 1.0f, 0.0f, 0.0f,
								0.0f, 0.0f, 1.0f, 0.0f,
								0.0f, 0.0f, 0.0f, 1.0f };

	memcpy(data, identity, 16*sizeof(float));
}

clf_matrix4x4 clf_matrix4x4::operator * (const clf_matrix4x4& m1)
{
	clf_matrix4x4 mProduct;

	mProduct[0]    = m1[0] * data[0] + m1[4] * data[1] + m1[8] * data[2] + m1[12] * data[3];
	mProduct[4]    = m1[0] * data[4] + m1[4] * data[5] + m1[8] * data[6] + m1[12] * data[7];
	mProduct[8]    = m1[0] * data[8] + m1[4] * data[9] + m1[8] * data[10] + m1[12] * data[11];
	mProduct[12]   = m1[0] * data[12] + m1[4] * data[13] + m1[8] * data[14] + m1[12] * data[15];

	mProduct[1]    = m1[1] * data[0] + m1[5] * data[1] + m1[9] * data[2] + m1[13] * data[3];
	mProduct[5]    = m1[1] * data[4] + m1[5] * data[5] + m1[9] * data[6] + m1[13] * data[7];
	mProduct[9]    = m1[1] * data[8] + m1[5] * data[9] + m1[9] * data[10] + m1[13] * data[11];
	mProduct[13]   = m1[1] * data[12] + m1[5] * data[13] + m1[9] * data[14] + m1[13] * data[15];

	mProduct[2]    = m1[2] * data[0] + m1[6] * data[1] + m1[10] * data[2] + m1[14] * data[3];
	mProduct[6]    = m1[2] * data[4] + m1[6] * data[5] + m1[10] * data[6] + m1[14] * data[7];
	mProduct[10]   = m1[2] * data[8] + m1[6] * data[9] + m1[10] * data[10] + m1[14] * data[11];
	mProduct[14]   = m1[2] * data[12] + m1[6] * data[13] + m1[10] * data[14] + m1[14] * data[15];

	mProduct[3]    = m1[3] * data[0] + m1[7] * data[1] + m1[11] * data[2] + m1[15] * data[3];
	mProduct[7]    = m1[3] * data[4] + m1[7] * data[5] + m1[11] * data[6] + m1[15] * data[7];
	mProduct[11]   = m1[3] * data[8] + m1[7] * data[9] + m1[11] * data[10] + m1[15] * data[11];
	mProduct[15]   = m1[3] * data[12] + m1[7] * data[13] + m1[11] * data[14] + m1[15] * data[15];

	return mProduct;
}

clf_vector4 clf_matrix4x4::operator *(clf_vector4& v)
{
	clf_vector4 mProduct;

	mProduct[0] = data[0]*v[0]+data[4]*v[1]+data[8]*v[2]+data[12]*v[3];
	mProduct[1] = data[1]*v[0]+data[5]*v[1]+data[9]*v[2]+data[13]*v[3];
	mProduct[2] = data[2]*v[0]+data[6]*v[1]+data[10]*v[2]+data[14]*v[3];
	mProduct[3] = data[3]*v[0]+data[7]*v[1]+data[11]*v[2]+data[15]*v[3];
	
	return mProduct;
}

void clf_matrix4x4::Transpose()
{
	float temp;

	temp     = data[ 1];
	data[ 1] = data[ 4];
	data[ 4] = temp;

	temp     = data[ 2];
	data[ 2] = data[ 8];
	data[ 8] = temp;

	temp     = data[ 3];
	data[ 3] = data[12];
	data[12] = temp;

	temp     = data[ 6];
	data[ 6] = data[ 9];
	data[ 9] = temp;

	temp     = data[ 7];
	data[ 7] = data[13];
	data[13] = temp;

	temp     = data[11];
	data[11] = data[14];
	data[14] = temp;
}

float clf_matrix4x4::DetIJ(int i, int j)
{
	int x, y, ii, jj;
	float ret, mat[3][3];

	x = 0;
	for (ii = 0; ii < 4; ii++)
	{
		if (ii == i) continue;
		y = 0;
		for (jj = 0; jj < 4; jj++)
		{
			if (jj == j) continue;
			mat[x][y] = data[(ii*4)+jj];
			y++;
		}
		x++;
	}

	ret =  mat[0][0]*(mat[1][1]*mat[2][2]-mat[2][1]*mat[1][2]);
	ret -= mat[0][1]*(mat[1][0]*mat[2][2]-mat[2][0]*mat[1][2]);
	ret += mat[0][2]*(mat[1][0]*mat[2][1]-mat[2][0]*mat[1][1]);

	return ret;
}

clf_matrix4x4& clf_matrix4x4::Invert()
{
	int i, j;
	float det, detij;
	float mInverse[16];

	// calculate 4x4 determinant
	det = 0.0f;
	for (i = 0; i < 4; i++)
	{
		det += (i & 0x1) ? (-data[i] * DetIJ(0, i)) : (data[i] * DetIJ(0,i));
	}
	det = 1.0f / det;

	// calculate inverse
	for (i = 0; i < 4; i++)
	{
		for (j = 0; j < 4; j++)
		{
			detij = DetIJ(j, i);
			mInverse[(i*4)+j] = ((i+j) & 0x1) ? (-detij * det) : (detij *det); 
		}
	}

	return *this;
}

//////////////////////////////////////////////////////////////////////////////
// Create a translation matrix
void clf_matrix4x4::TranslationMatrix(float x, float y, float z)
{
	loadIdentity();
	data[12] = x;
	data[13] = y;
	data[14] = z;
}

///////////////////////////////////////////////////////////////////////////////
// Create a scaling matrix
void clf_matrix4x4::ScalingMatrix(float x, float y, float z)
{
	loadIdentity();
	data[0] = x;
	data[5] = y;
	data[11] = z;
}

///////////////////////////////////////////////////////////////////////////////
// Creates a 4x4 rotation matrix, takes radians NOT degrees
void clf_matrix4x4::RotationMatrix(float angle, float x, float y, float z)
{
	float vecLength, sinSave, cosSave, oneMinusCos;
	float xx, yy, zz, xy, yz, zx, xs, ys, zs;

	if(x == 0.0f && y == 0.0f && z == 0.0f)
	{
		loadIdentity();
		return;
	}

	// Scale vector
	vecLength = (float)sqrt( x*x + y*y + z*z );

	// Rotation matrix is normalized
	x /= vecLength;
	y /= vecLength;
	z /= vecLength;

	sinSave = (float)mySin(angle);
	cosSave = (float)myCos(angle);
	oneMinusCos = 1.0f - cosSave;

	xx = x * x;
	yy = y * y;
	zz = z * z;
	xy = x * y;
	yz = y * z;
	zx = z * x;
	xs = x * sinSave;
	ys = y * sinSave;
	zs = z * sinSave;

	data[0] = (oneMinusCos * xx) + cosSave;
	data[4] = (oneMinusCos * xy) - zs;
	data[8] = (oneMinusCos * zx) + ys;
	data[12] = 0.0f;

	data[1] = (oneMinusCos * xy) + zs;
	data[5] = (oneMinusCos * yy) + cosSave;
	data[9] = (oneMinusCos * yz) - xs;
	data[13] = 0.0f;

	data[2] = (oneMinusCos * zx) - ys;
	data[6] = (oneMinusCos * yz) + xs;
	data[10] = (oneMinusCos * zz) + cosSave;
	data[14] = 0.0f;

	data[3] = 0.0f;
	data[7] = 0.0f;
	data[11] = 0.0f;
	data[15] = 1.0f;
}

// Creates a shadow projection matrix out of the plane equation
// coefficients and the position of the light. The return value is stored
// in destMat
void clf_matrix4x4::MakeShadowMatrix(clf_vector3D vPoints[3], clf_vector4& vLightPos)
{
	clf_vector4 planeEq;
	clf_getPlaneEquation(vPoints[0], vPoints[1], vPoints[2], planeEq);

	float a = planeEq[0];
	float b = planeEq[1];
	float c = planeEq[2];
	float d = planeEq[3];

	float dx = vLightPos[0];
	float dy = vLightPos[1];
	float dz = vLightPos[2];

	// Now build the projection matrix
	data[0] = b * dy + c * dz;
	data[1] = -a * dy;
	data[2] = -a * dz;
	data[3] = 0.0;

	data[4] = -b * dx;
	data[5] = a * dx + c * dz;
	data[6] = -b * dz;
	data[7] = 0.0;

	data[8] = -c * dx;
	data[9] = -c * dy;
	data[10] = a * dx + b * dy;
	data[11] = 0.0;

	data[12] = -d * dx;
	data[13] = -d * dy;
	data[14] = -d * dz;
	data[15] = a * dx + b * dy + c * dz;
}

void clf_matrix4x4::RotationMatrix(float angle, clf_vector3D& vAxis)
{
	float st = sin(angle);
	float ct = cos(angle);

	float u = vAxis.x;
	float v = vAxis.y;
	float w = vAxis.z;


	data[0] = u*u + (v*v + w*w)*ct;
	data[1] = u*v*(1-ct) - w*st;
	data[2] = u*w*(1-ct) + v*st;
	data[3] = 0;

	data[4] = u*v*(1-ct) + w*st;
	data[5] = v*v + (u*u + w*w)*ct;
	data[6] = v*w*(1-ct) - u*st;
	data[7] = 0;

	data[8] = u*w*(1-ct) - v*st;
	data[9] = v*w*(1-ct) + u*st;
	data[10] = w*w + (v*v + u*u)*ct;
	data[11] = 0;

	data[12] = 0;
	data[13] = 0;
	data[14] = 0;
	data[15] = 1;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Given three points on a plane in counter clockwise order, calculate the unit normal
void clf_getNormalVector(clf_vector3D& vP1, clf_vector3D& vP2, clf_vector3D& vP3, clf_vector3D& vNormal)
{
	clf_vector3D vV1, vV2;

	vV1 = vP2 - vP1;
	vV2 = vP3 - vP2;

	vNormal = crossProduct(vV1, vV2);
	vNormal.normalize();
}

// Transform a point by a 4x4 matrix
void clf_transformPoint(const clf_vector3D& vSrcVector, const clf_matrix4x4& mMatrix, clf_vector3D& vOut)
{
	vOut.x = mMatrix[0] * vSrcVector.x + mMatrix[4] * vSrcVector.y + mMatrix[8]  * vSrcVector.z + mMatrix[12];
	vOut.y = mMatrix[1] * vSrcVector.x + mMatrix[5] * vSrcVector.y + mMatrix[9]  * vSrcVector.z + mMatrix[13];
	vOut.z = mMatrix[2] * vSrcVector.x + mMatrix[6] * vSrcVector.y + mMatrix[10] * vSrcVector.z + mMatrix[14];    
}

// Rotates a vector using a 4x4 matrix. Translation column is ignored
void clf_rotateVector(const clf_vector3D& vSrcVector, const clf_matrix4x4& mMatrix, clf_vector3D& vOut)
{
	float x = mMatrix[0] * vSrcVector.x + mMatrix[4] * vSrcVector.y + mMatrix[8] *  vSrcVector.z;
	float y = mMatrix[1] * vSrcVector.x + mMatrix[5] * vSrcVector.y + mMatrix[9] *  vSrcVector.z;
	float z = mMatrix[2] * vSrcVector.x + mMatrix[6] * vSrcVector.y + mMatrix[10] * vSrcVector.z;    	

	vOut = clf_vector3D(x,y,z);
}

void clf_unrotateVector(const clf_vector3D& vSrcVector, const clf_matrix4x4& mMatrix, clf_vector3D& vOut)
{
	vOut.x = mMatrix[0] * vSrcVector.x + mMatrix[1] * vSrcVector.y + mMatrix[2] *  vSrcVector.z;
	vOut.y = mMatrix[4] * vSrcVector.x + mMatrix[5] * vSrcVector.y + mMatrix[6] *  vSrcVector.z;
	vOut.z = mMatrix[8] * vSrcVector.x + mMatrix[9] * vSrcVector.y + mMatrix[10] * vSrcVector.z;    	
	// return dVector (v % m_front, v % m_up, v % m_right);
}
/*
T TemplateVector<T>::operator% (const TemplateVector<T> &A) const
{
	return m_x * A.m_x + m_y * A.m_y + m_z * A.m_z;
}
*/

// Gets the three coefficients of a plane equation given three points on the plane.
void clf_getPlaneEquation(clf_vector3D& vPoint1, clf_vector3D& vPoint2, clf_vector3D& vPoint3, clf_vector4& vPlane)
{
	// Get normal vector from three points. The normal vector is the first three coefficients
	// to the plane equation...
	clf_vector3D norm;
	clf_getNormalVector(vPoint1, vPoint2, vPoint3, norm);

	vPlane[0] = norm.x;
	vPlane[1] = norm.y;
	vPlane[2] = norm.z;

	// Final coefficient found by back substitution
	vPlane[3] = -(vPlane[0] * vPoint3.x + vPlane[1] * vPoint3.y + vPlane[2] * vPoint3.z);
}

// Determine the distance of a point from a plane, given the point and the
// equation of the plane.
float clf_distanceToPlane(clf_vector3D& vPoint, clf_vector4& vPlane)
{
	return vPoint.x*vPlane[0] + vPoint[1]*vPlane[1] + vPoint[2]*vPlane[2] + vPlane[3];
}

void rotate_point_rad(float x, float y, float z, clf_vector3D& point)
{
	float sy,cy;
	float sx,cx;
	float sz,cz;

	sy = sin(y);
	cy = cos(y);
	sx = sin(x);
	cx = cos(x);
	sz = sin(z);
	cz = cos(z);

	clf_vector3D np(point);

	clf_vector3D np1(point);   // around Y
    np1.x = np.z*sy + np.x*cy;
	np1.y = np.y;
	np1.z = np.z*cy - np.x*sy;

	clf_vector3D np2 = np1;	// around X
	np2.x = np1.x;
	np2.y = np1.y*cx - np1.z*sx;
	np2.z = np1.y*sx + np1.z*cx;

	point.x = np2.y*sz + np2.x*cz;
	point.y = np2.y*cz - np2.x*sz;
	point.z = np2.z;
}

void rotate_point(float x, float y, float z, clf_vector3D& point)
{
	float sy,cy;
	float sx,cx;
	float sz,cz;

	sy = (float)mySin(y);
	cy = (float)myCos(y);
	sx = (float)mySin(x);
	cx = (float)myCos(x);
	sz = (float)mySin(z);
	cz = (float)myCos(z);

	clf_vector3D np(point);

	clf_vector3D np1(point);   // around Y
  np1.x = np.z*sy + np.x*cy;
	np1.y = np.y;
	np1.z = np.z*cy - np.x*sy;

	clf_vector3D np2 = np1;	// around X
	np2.x = np1.x;
	np2.y = np1.y*cx - np1.z*sx;
	np2.z = np1.y*sx + np1.z*cx;

	point.x = np2.y*sz + np2.x*cz;
	point.y = np2.y*cz - np2.x*sz;
	point.z = np2.z;
}

void rotate_f(float x, float y, float z, clf_vector3D& point)
{
	float sy,cy;
	float sx,cx;
	float sz,cz;

	sy = x;
	cy = z;
	sx = z;
	cx = y;
	sz = y;
	cz = x;

	clf_vector3D np(point);

	clf_vector3D np1(point);   // around Y
  np1.x = np.z*sy + np.x*cy;
	np1.y = np.y;
	np1.z = np.z*cy - np.x*sy;

	clf_vector3D np2 = np1;	// around X
	np2.x = np1.x;
	np2.y = np1.y*cx - np1.z*sx;
	np2.z = np1.y*sx + np1.z*cx;

	point.x = np2.y*sz + np2.x*cz;
	point.y = np2.y*cz - np2.x*sz;
	point.z = np2.z;
}
///////////////////////////////////////////////////////////////////////////////
// library init

void init_math_lib()
{
	init_trig();
}
